@@ -1,5 +1,8 @@
+#![deny(missing_docs)]
 //! A lossless parser for Debian changelog files.
 //!
+//! See https://manpages.debian.org/bookworm/dpkg-dev/deb-changelog.5.en.html
+//!
 //! For its format specification, see [Debian Policy](https://www.debian.org/doc/debian-policy/ch-source.html#debian-changelog-debian-changelog).
 //!
 //! Example:
@@ -15,7 +18,7 @@
 //! let changelog: debian_changelog::ChangeLog = contents.parse().unwrap();
 //! assert_eq!(
 //!     vec![("rustc".to_string(), "1.70.0+dfsg1-1".parse().unwrap())],
-//!     changelog.entries().map(
+//!     changelog.iter().map(
 //!         |e| (e.package().unwrap(), e.version().unwrap()))
 //!     .collect::<Vec<_>>());
 //! ```
@@ -28,13 +31,12 @@ pub mod textwrap;
 
 pub use crate::parse::{ChangeLog, Entry, Error, ParseError, Urgency};
 
-/// See https://manpages.debian.org/bookworm/dpkg-dev/deb-changelog.5.en.html
-
 /// Let's start with defining all kinds of tokens and
 /// composite nodes.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 #[allow(non_camel_case_types)]
 #[repr(u16)]
+#[allow(missing_docs)]
 pub enum SyntaxKind {
     IDENTIFIER = 0,
     INDENT,
@@ -92,6 +94,7 @@ pub fn parseaddr(s: &str) -> (Option<&str>, &str) {
     }
 }
 
+/// Get the maintainer information from the environment.
 pub fn get_maintainer_from_env(
     get_env: impl Fn(&str) -> Option<String>,
 ) -> Option<(String, String)> {
@@ -234,6 +237,7 @@ pub struct Identity {
 }
 
 impl Identity {
+    /// Create a new identity.
     pub fn new(name: String, email: String) -> Self {
         Self { name, email }
     }
@@ -279,20 +283,19 @@ fn test_distributions_is_unreleased() {
 
 /// Check whether this is a traditional inaugural release
 pub fn is_unreleased_inaugural(cl: &ChangeLog) -> bool {
-    let mut entries = cl.entries();
-    match entries.next() {
-        None => return false,
-        Some(entry) => {
-            if entry.is_unreleased() == Some(false) {
-                return false;
-            }
-            let changes = entry.change_lines().collect::<Vec<_>>();
-            if changes.len() > 1 || !changes[0].starts_with("* Initial release") {
-                return false;
-            }
+    let mut entries = cl.iter();
+    if let Some(entry) = entries.next() {
+        if entry.is_unreleased() == Some(false) {
+            return false;
         }
+        let changes = entry.change_lines().collect::<Vec<_>>();
+        if changes.len() > 1 || !changes[0].starts_with("* Initial release") {
+            return false;
+        }
+        entries.next().is_none()
+    } else {
+        false
     }
-    entries.next().is_none()
 }
 
 #[cfg(test)]
@@ -325,7 +328,7 @@ mod is_unreleased_inaugural_tests {
             .version("1.0.0".parse().unwrap())
             .change_line("* Initial release".to_string())
             .finish();
-        assert_eq!(cl.entries().next().unwrap().is_unreleased(), Some(false));
+        assert_eq!(cl.iter().next().unwrap().is_unreleased(), Some(false));
 
         // Not unreleased
         assert!(!is_unreleased_inaugural(&cl));
@@ -361,7 +364,7 @@ pub fn release(
     timestamp: Option<chrono::DateTime<chrono::FixedOffset>>,
     maintainer: Option<(String, String)>,
 ) -> bool {
-    let mut entries = cl.entries();
+    let mut entries = cl.iter();
     let mut first_entry = entries.next().unwrap();
     let second_entry = entries.next();
     let distribution = if let Some(d) = distribution.as_ref() {
