@@ -7,12 +7,22 @@ use rowan::ast::AstNode;
 use std::str::FromStr;
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, Default, PartialOrd, Ord)]
+/// Urgency of the changes in the changelog entry
 pub enum Urgency {
     #[default]
+    /// Low urgency
     Low,
+
+    /// Medium urgency
     Medium,
+
+    /// High urgency
     High,
+
+    /// Emergency urgency
     Emergency,
+
+    /// Critical urgency
     Critical,
 }
 
@@ -44,8 +54,12 @@ impl FromStr for Urgency {
 }
 
 #[derive(Debug)]
+/// Error while reading a changelog file.
 pub enum Error {
+    /// I/O Error
     Io(std::io::Error),
+
+    /// Parsing error
     Parse(ParseError),
 }
 
@@ -67,6 +81,7 @@ impl From<std::io::Error> for Error {
 impl std::error::Error for Error {}
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
+/// Error while parsing
 pub struct ParseError(Vec<String>);
 
 impl std::fmt::Display for ParseError {
@@ -411,11 +426,11 @@ fn parse(text: &str) -> Parse {
     .parse()
 }
 
-/// To work with the parse results we need a view into the
-/// green tree - the Syntax tree.
-/// It is also immutable, like a GreenNode,
-/// but it contains parent pointers, offsets, and
-/// has identity semantics.
+// To work with the parse results we need a view into the
+// green tree - the Syntax tree.
+// It is also immutable, like a GreenNode,
+// but it contains parent pointers, offsets, and
+// has identity semantics.
 
 type SyntaxNode = rowan::SyntaxNode<Lang>;
 #[allow(unused)]
@@ -438,6 +453,7 @@ macro_rules! ast_node {
     ($ast:ident, $kind:ident) => {
         #[derive(PartialEq, Eq, Hash)]
         #[repr(transparent)]
+        /// A node in the changelog syntax tree.
         pub struct $ast(SyntaxNode);
 
         impl AstNode for $ast {
@@ -725,7 +741,43 @@ impl EntryBuilder {
     }
 }
 
+impl IntoIterator for ChangeLog {
+    type Item = Entry;
+    type IntoIter = std::vec::IntoIter<Entry>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        // TODO: This is inefficient
+        self.iter().collect::<Vec<_>>().into_iter()
+    }
+}
+
+fn replay(builder: &mut GreenNodeBuilder, node: SyntaxNode) {
+    builder.start_node(node.kind().into());
+    for child in node.children_with_tokens() {
+        match child {
+            SyntaxElement::Node(n) => replay(builder, n),
+            SyntaxElement::Token(t) => {
+                builder.token(t.kind().into(), t.text());
+            }
+        }
+    }
+    builder.finish_node();
+}
+
+impl FromIterator<Entry> for ChangeLog {
+    fn from_iter<T: IntoIterator<Item = Entry>>(iter: T) -> Self {
+        let mut builder = GreenNodeBuilder::new();
+        builder.start_node(ROOT.into());
+        for entry in iter {
+            replay(&mut builder, entry.0.clone());
+        }
+        builder.finish_node();
+        ChangeLog(SyntaxNode::new_root_mut(builder.finish()))
+    }
+}
+
 impl ChangeLog {
+    /// Create a new, empty changelog.
     pub fn new() -> ChangeLog {
         let mut builder = GreenNodeBuilder::new();
 
@@ -736,11 +788,18 @@ impl ChangeLog {
         ChangeLog(syntax)
     }
 
-    /// Returns an iterator over all entries in the watch file.
-    pub fn entries(&self) -> impl Iterator<Item = Entry> + '_ {
+    /// Returns an iterator over all entries in the changelog file.
+    pub fn iter(&self) -> impl Iterator<Item = Entry> + '_ {
         self.0.children().filter_map(Entry::cast)
     }
 
+    /// Returns an iterator over all entries in the changelog file.
+    #[deprecated(since = "0.2.0", note = "use `iter` instead")]
+    pub fn entries(&self) -> impl Iterator<Item = Entry> + '_ {
+        self.iter()
+    }
+
+    /// Create a new, empty entry.
     pub fn new_empty_entry(&mut self) -> EntryBuilder {
         EntryBuilder {
             root: self.0.clone(),
@@ -755,7 +814,7 @@ impl ChangeLog {
     }
 
     fn first_valid_entry(&self) -> Option<Entry> {
-        self.entries().find(|entry| {
+        self.iter().find(|entry| {
             entry.package().is_some() && entry.header().is_some() && entry.footer().is_some()
         })
     }
@@ -832,8 +891,9 @@ impl ChangeLog {
         }
     }
 
+    /// Pop the first entry from the changelog.
     pub fn pop_first(&mut self) -> Option<Entry> {
-        let mut it = self.entries();
+        let mut it = self.iter();
         if let Some(entry) = it.next() {
             // Drop trailing newlines
             while let Some(sibling) = entry.0.next_sibling() {
@@ -863,6 +923,7 @@ impl ChangeLog {
         Ok(buf.parse()?)
     }
 
+    /// Read a changelog file from a reader, allowing for syntax errors
     pub fn read_relaxed<R: std::io::Read>(mut r: R) -> Result<ChangeLog, Error> {
         let mut buf = String::new();
         r.read_to_string(&mut buf)?;
@@ -871,12 +932,14 @@ impl ChangeLog {
         Ok(parsed.root_mut())
     }
 
+    /// Write the changelog to a writer
     pub fn write<W: std::io::Write>(&self, mut w: W) -> Result<(), Error> {
         let buf = self.to_string();
         w.write_all(buf.as_bytes())?;
         Ok(())
     }
 
+    /// Write the changelog to a path
     pub fn write_to_path(&self, p: &std::path::Path) -> Result<(), Error> {
         let f = std::fs::File::create(p)?;
         self.write(f)?;
@@ -908,7 +971,7 @@ impl FromStr for Entry {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let cl: ChangeLog = s.parse()?;
-        let mut entries = cl.entries();
+        let mut entries = cl.iter();
         let entry = entries
             .next()
             .ok_or_else(|| ParseError(vec!["no entries found".to_string()]))?;
@@ -1408,6 +1471,7 @@ impl Entry {
         self.header().and_then(|h| h.package())
     }
 
+    /// Set the package name of the entry.
     pub fn set_package(&mut self, package: String) {
         self.header()
             .unwrap_or_else(|| self.create_header())
@@ -1419,6 +1483,7 @@ impl Entry {
         self.header().and_then(|h| h.version())
     }
 
+    /// Set the version of the entry.
     pub fn set_version(&mut self, version: &Version) {
         self.header()
             .unwrap_or_else(|| self.create_header())
@@ -1430,6 +1495,7 @@ impl Entry {
         self.header().and_then(|h| h.distributions())
     }
 
+    /// Set the distributions for the entry
     pub fn set_distributions(&mut self, distributions: Vec<String>) {
         self.header()
             .unwrap_or_else(|| self.create_header())
@@ -1446,6 +1512,7 @@ impl Entry {
         self.footer().and_then(|f| f.maintainer())
     }
 
+    /// Set the maintainer of the entry.
     pub fn set_maintainer(&mut self, maintainer: (String, String)) {
         let mut footer = self.footer().unwrap_or_else(|| self.create_footer());
 
@@ -1458,12 +1525,14 @@ impl Entry {
         self.footer().and_then(|f| f.timestamp())
     }
 
+    /// Set the timestamp of the entry.
     pub fn set_timestamp(&mut self, timestamp: String) {
         self.footer()
             .unwrap_or_else(|| self.create_footer())
             .set_timestamp(timestamp);
     }
 
+    /// Set the datetime of the entry.
     pub fn set_datetime(&mut self, datetime: DateTime<FixedOffset>) {
         self.set_timestamp(format!("{}", datetime.format("%a, %d %b %Y %H:%M:%S %z")));
     }
@@ -1499,10 +1568,12 @@ impl Entry {
         EntryFooter(self.0.children().last().unwrap().clone_for_update())
     }
 
+    /// Set the urgency of the entry.
     pub fn set_urgency(&mut self, urgency: Urgency) {
         self.set_metadata("urgency", urgency.to_string().as_str());
     }
 
+    /// Set a metadata key-value pair for the entry.
     pub fn set_metadata(&mut self, key: &str, value: &str) {
         self.header()
             .unwrap_or_else(|| self.create_header())
@@ -1553,6 +1624,7 @@ impl Entry {
         }
     }
 
+    /// Prepend a change line to the entry
     pub fn prepend_change_line(&self, line: &str) {
         let mut builder = GreenNodeBuilder::new();
         builder.start_node(ENTRY_BODY.into());
@@ -1576,6 +1648,7 @@ impl Entry {
         self.0.splice_children(index..index, vec![syntax.into()]);
     }
 
+    /// Pop the last change line from the entry
     pub fn pop_change_line(&self) -> Option<String> {
         // Find the last child of type ENTRY_BODY
         let last_child = self.0.children().filter(|n| n.kind() == ENTRY_BODY).last();
@@ -1597,6 +1670,7 @@ impl Entry {
         }
     }
 
+    /// Append a line to the changelog entry
     pub fn append_change_line(&self, line: &str) {
         let mut builder = GreenNodeBuilder::new();
         builder.start_node(ENTRY_BODY.into());
@@ -1647,6 +1721,9 @@ impl Entry {
         lines.into_iter().skip_while(|it| it.is_empty())
     }
 
+    /// Ensure that the first line of the entry is the specified line
+    ///
+    /// If the first line is not the specified line, it will be prepended to the entry.
     pub fn ensure_first_line(&self, line: &str) {
         let first_line = self.change_lines().next().map(|it| it.trim().to_string());
 
@@ -1820,7 +1897,7 @@ breezy (3.3.3-2) unstable; urgency=medium
         );
 
         let mut root = parsed.root_mut();
-        let entries: Vec<_> = root.entries().collect();
+        let entries: Vec<_> = root.iter().collect();
         assert_eq!(entries.len(), 2);
         let entry = &entries[0];
         assert_eq!(entry.package(), Some("breezy".into()));
@@ -1895,7 +1972,7 @@ breezy (3.3.3-2) unstable; urgency=medium
             cl.to_string()
         );
 
-        assert!(!cl.entries().next().unwrap().is_unreleased().unwrap());
+        assert!(!cl.iter().next().unwrap().is_unreleased().unwrap());
     }
 
     #[test]
@@ -1934,7 +2011,7 @@ breezy (3.3.3-2) unstable; urgency=medium
 "###,
             cl.to_string()
         );
-        assert_eq!(cl.entries().next().unwrap().is_unreleased(), Some(true));
+        assert_eq!(cl.iter().next().unwrap().is_unreleased(), Some(true));
     }
 
     #[test]
@@ -1948,7 +2025,7 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
  -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
 "#;
         let cl = ChangeLog::read_relaxed(text.as_bytes()).unwrap();
-        let entry = cl.entries().nth(1).unwrap();
+        let entry = cl.iter().nth(1).unwrap();
         assert_eq!(entry.package(), Some("lintian-brush".into()));
         assert_eq!(entry.version(), Some("0.35".parse().unwrap()));
         assert_eq!(entry.urgency(), Some(Urgency::Medium));
@@ -2033,7 +2110,7 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
 "#;
             let mut cl = super::ChangeLog::read(text.as_bytes()).unwrap();
 
-            let entry = cl.entries().next().unwrap();
+            let entry = cl.iter().next().unwrap();
             assert_eq!(entry.package(), Some("lintian-brush".into()));
             assert_eq!(entry.is_unreleased(), Some(true));
 
@@ -2044,7 +2121,7 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
                 None,
             );
 
-            assert_eq!(cl.entries().count(), 1);
+            assert_eq!(cl.iter().count(), 1);
 
             assert_eq!(entry.package(), Some("lintian-brush".into()));
             assert_eq!(entry.is_unreleased(), Some(true));
@@ -2076,7 +2153,7 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
 "#;
         let cl = ChangeLog::read(text.as_bytes()).unwrap();
 
-        let entry = cl.entries().next().unwrap();
+        let entry = cl.iter().next().unwrap();
         assert_eq!(entry.package(), Some("lintian-brush".into()));
 
         entry.ensure_first_line("* QA upload.");
@@ -2308,4 +2385,20 @@ lintian-brush (0.35) UNRELEASED; urgency=medium
             ("Jelmer Vernooĳ".into(), "jelmer@debian.org".into()),
         );
     }
+
+    #[test]
+    fn test_changelog_from_entry_iter() {
+        let text = r#"breezy (3.3.4-1) unstable; urgency=low
+
+  * New upstream release.
+
+ -- Jelmer Vernooĳ <jelmer@jelmer.uk>  Mon, 04 Sep 2023 18:13:45 -0500
+"#;
+
+        let entry: Entry = text.parse().unwrap();
+
+        let cl = std::iter::once(entry).collect::<ChangeLog>();
+
+        assert_eq!(cl.to_string(), text);
+    }
 }
